"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var react_table_plain_1 = require("@dccs/react-table-plain");
var DataGridStateContext_1 = require("./DataGridStateContext");
DataGridPlain.defaultProps = {
    initialLoad: true
};
function DataGridPlain(props) {
    // Internal state...if there is no DataGridStateProvider.
    var _a = React.useState(props.initialRowsPerPage || 10), rowsPerPage = _a[0], setRowsPerPage = _a[1];
    var _b = React.useState(0), page = _b[0], setPage = _b[1];
    var _c = React.useState(props.initialLoad === true ? true : false), allowLoad = _c[0], setAllowLoad = _c[1];
    var _d = React.useState(0), total = _d[0], setTotal = _d[1];
    var _e = React.useState(props.initialOrderBy), orderBy = _e[0], setOrderBy = _e[1];
    var _f = React.useState(), sort = _f[0], setSort = _f[1];
    var _g = React.useState(), filter = _g[0], setFilter = _g[1];
    // Do we have a custom context,...
    var state = React.useContext(DataGridStateContext_1.DataGridState) || {
        rowsPerPage: rowsPerPage,
        setRowsPerPage: setRowsPerPage,
        page: page,
        setPage: setPage,
        total: total,
        setTotal: setTotal,
        orderBy: orderBy,
        setOrderBy: setOrderBy,
        sort: sort,
        setSort: setSort,
        filter: props.filter || filter,
        setFilter: setFilter,
        reloadDummy: false,
        reload: function () {
            throw new Error("No supported in DateGridPlain, only in DataGridStateProvider.");
        }
    };
    var _h = React.useState([]), data = _h[0], setData = _h[1];
    var _j = React.useState(false), loading = _j[0], setLoading = _j[1];
    var _k = React.useState(false), error = _k[0], setError = _k[1];
    React.useEffect(function () {
        if (allowLoad === true) {
            load();
        }
        else {
            // allowLoad on second try.
            setAllowLoad(true);
        }
    }, [
        state.rowsPerPage,
        state.page,
        state.orderBy,
        state.sort === "desc",
        state.reloadDummy,
        props.onLoadData,
        // Why JSON.stringify?
        // The way the useEffect dependency array works is by checking for strict (===) equivalency between all of the items in the array from the previous render and the new render.
        // Example:  {}==={}                                   -> false -> different -> rerender
        // Example2: JSON.stringify({}) === JSON.stringify({}) -> true  -> same      -> no rerender
        JSON.stringify(state.filter)
    ]);
    function load() {
        setLoading(true);
        setError(false);
        props
            .onLoadData(state.page + 1, state.rowsPerPage, state.orderBy, state.sort, state.filter)
            .then(function (_a) {
            var d = _a.data, t = _a.total;
            state.setTotal(t);
            setData(d);
            setLoading(false);
        })
            .catch(function () {
            setLoading(false);
            setError(true);
        });
    }
    function handleChangePage(p) {
        state.setPage(p);
    }
    function handleChangeRowsPerPage(rows) {
        state.setRowsPerPage(rows);
    }
    function handleChangeOrderBy(ob) {
        var s;
        if (state.orderBy && state.orderBy === ob) {
            s = state.sort === "desc" ? "asc" : "desc";
        }
        state.setOrderBy(ob);
        state.setSort(s);
    }
    function handleChangeFilter(ob, value) {
        var _a;
        if (props.onChangeFilter) {
            props.onChangeFilter(ob, value);
        }
        else {
            state.setFilter(__assign({}, state.filter, (_a = {}, _a[ob] = value, _a)));
        }
    }
    function renderTable() {
        var ps = {
            data: data,
            colDef: props.colDef,
            orderBy: state.orderBy,
            sort: state.sort,
            filter: state.filter,
            onChangeOrderBy: handleChangeOrderBy,
            onChangeFilter: handleChangeFilter,
            onRowClick: props.onRowClick,
            subComponent: props.subComponent,
            selectedRow: props.selectedRow,
            selectedRowProps: props.selectedRowProps,
            onChangeSelectedRow: props.onChangeSelectedRow,
            rowSelectionColumnName: props.rowSelectionColumnName
        };
        if (props.renderTable != null) {
            return props.renderTable(ps);
        }
        return React.createElement(react_table_plain_1.TablePlain, __assign({}, ps));
    }
    function renderLoading() {
        if (props.renderLoading != null) {
            return props.renderLoading();
        }
        var loadingText = "Loading...";
        if (props.texts && props.texts.loadingText != null) {
            loadingText = props.texts.loadingText;
        }
        return React.createElement("h5", null, loadingText);
    }
    function renderError() {
        var errorText = "Die Daten konnten nicht geladen werden.";
        if (props.texts && props.texts.errorText != null) {
            errorText = props.texts.errorText;
        }
        var reloadText = "Neu laden";
        if (props.texts && props.texts.reloadText != null) {
            reloadText = props.texts.reloadText;
        }
        if (props.renderError != null) {
            return props.renderError(load, errorText, reloadText);
        }
        return (React.createElement("p", { style: { width: "100%", background: "red", padding: 16 } },
            React.createElement("p", null, errorText),
            React.createElement("p", null,
                React.createElement("button", { onClick: function () { return load(); } }, reloadText))));
    }
    function renderPaging() {
        if (props.renderPaging != null) {
            var labelRowsPerPage = props.texts && props.texts.labelRowsPerPage;
            var backIconButtonText = props.texts && props.texts.backIconButtonText;
            var nextIconButtonText = props.texts && props.texts.nextIconButtonText;
            var labelDisplayedRows = props.texts && props.texts.labelDisplayedRows;
            return props.renderPaging(__assign({}, state, { labelRowsPerPage: labelRowsPerPage, backIconButtonText: backIconButtonText, nextIconButtonText: nextIconButtonText, labelDisplayedRows: labelDisplayedRows, handleChangePage: handleChangePage,
                handleChangeRowsPerPage: handleChangeRowsPerPage }));
        }
        var pagingText = "No paging available right now. Check back later...";
        if (props.texts && props.texts.pagingText != null) {
            pagingText = props.texts.pagingText;
        }
        return React.createElement("strong", null, pagingText);
    }
    if (error) {
        return renderError();
    }
    return (React.createElement("div", { style: { position: "relative", overflowX: "auto", width: "100%" } },
        loading && renderLoading(),
        renderTable(),
        props.disablePaging !== true && renderPaging()));
}
exports.DataGridPlain = DataGridPlain;
